from PDFContentConverter import PDFContentConverter
import networkx as nx
import numpy as np
from GraphConverter.util import RectangleUtil, constants
from GraphConverter.document.DocumentMetaCharacteristics import DocumentMetaCharacteristics
from GraphConverter.merging.PDFTextBoxMerging import PDFTextBoxMerging


class GraphConverter(object):

    def __init__(self, pdf, merge_boxes=False, regress_parameters=False,
                 use_font=True, use_width=True, use_rect=True, use_horizontal_overlap=False,
                 use_vertical_overlap=False,
                 page_ratio_x=2, page_ratio_y=2, x_eps=2, y_eps=2, font_eps_h=1, font_eps_v=1,
                 width_pct_eps=.4, width_page_eps=.5):
        self.pdf = pdf
        self.set_attributes(merge_boxes, regress_parameters,
                            use_font, use_width, use_rect, use_horizontal_overlap, use_vertical_overlap,
                            page_ratio_x, page_ratio_y, x_eps, y_eps, font_eps_h, font_eps_v,
                            width_pct_eps, width_page_eps)
        conv = PDFContentConverter(self.pdf).convert()
        self.loc_df = conv["content"]
        self.media_boxes = conv["media_boxes"]
        self.n = conv["page_count"]
        self.loc_df = PDFTextBoxMerging(data=self.loc_df,
                                        media_boxes=self.media_boxes).transform()
        if self.loc_df is not None:
            self.meta = DocumentMetaCharacteristics(self.loc_df, self.media_boxes,
                                                    optimize=regress_parameters)
            self.meta.generate_attributes()

    def set_attributes(self, merge_boxes, regress_parameters,
                       use_font, use_width, use_rect, use_horizontal_overlap, use_vertical_overlap,
                       page_ratio_x, page_ratio_y, x_eps, y_eps, font_eps_h, font_eps_v,
                       width_pct_eps, width_page_eps):
        """

        :param merge_boxes:
        :param regress_parameters:
        :param use_font:
        :param use_width:
        :param use_rect:
        :param use_horizontal_overlap:
        :param use_vertical_overlap:
        :param page_ratio_x:
        :param page_ratio_y:
        :param x_eps:
        :param y_eps:
        :param font_eps_h:
        :param font_eps_v:
        :param width_pct_eps:
        :param width_page_eps:
        :return:
        """
        constants.USE_FONT = use_font
        constants.USE_WIDTH = use_width
        constants.USE_RECT = use_rect
        constants.USE_HORIZONTAL_OVERLAP = use_horizontal_overlap
        constants.USE_VERTICAL_OVERLAP = use_vertical_overlap
        constants.MERGE_RECTANGLE_TEXT_BOXES = merge_boxes
        constants.USE_RECTANGLE_BOX_COORDINATES = merge_boxes
        constants.OPTIMIZE = regress_parameters
        constants.DEFAULT_PAGE_RATIO_X = page_ratio_x
        constants.DEFAULT_PAGE_RATIO_Y = page_ratio_y
        constants.DEFAULT_X_EPS = x_eps
        constants.DEFAULT_Y_EPS = y_eps
        constants.DEFAULT_X_PHYS = x_eps
        constants.DEFAULT_FONT_EPS_HORIZONTAL = font_eps_h
        constants.DEFAULT_FONT_EPS_VERTICAL = font_eps_v
        constants.DEFAULT_WIDTH_PCT = width_pct_eps
        constants.DEFAULT_PAGE_WIDTH_EPS = width_page_eps

    def convert(self):
        """

        :return:
        """
        if self.loc_df is None:
            return [], None

        self.loc_df.set_index("id", inplace=True)
        self.loc_df["cluster_label"] = 0

        graph_list = []
        for page in range(self.meta.num_pages):
            G = nx.MultiDiGraph()
            loc_df_page = self.loc_df[self.loc_df["page"] == page]
            G.add_nodes_from(list(loc_df_page.index))
            nodes = loc_df_page.to_dict("index")
            nx.set_node_attributes(G, nodes)
            G = self.add_edges(G, page)
            G = nx.disjoint_union(nx.MultiDiGraph(), G)
            G = self.label_loops(G)
            graph_list.append(G)
        return {"graphs": graph_list,
                "document_meta": self.meta}

    def add_edges(self, G, page=0):
        """

        :param G:
        :param page:
        :return:
        """
        nodes = dict(G.nodes(data=True))
        node_ids = list(nodes.keys())
        for i in range(len(node_ids) - 1):
            id1 = node_ids[i]
            node1 = nodes[id1]

            for j in range(i + 1, len(node_ids)):
                id2 = node_ids[j]
                node2 = nodes[id2]

                delta_x_min = min(abs(node2["x_0"] - node1["x_0"]),
                                  abs(node2["x_1"] - node1["x_1"]),
                                  abs(node2["pos_x"] - node1["pos_x"]))
                delta_y_min = min(abs(node2["y_0"] - node1["y_0"]),
                                  abs(node2["y_1"] - node1["y_1"]),
                                  abs(node2["pos_y"] - node1["pos_y"]))
                length_x_min = min(abs(node2["x_1"] - node1["x_0"]),
                                   abs(node2["x_0"] - node1["x_1"]))
                length_y_min = min(abs(node2["y_1"] - node1["y_0"]),
                                   abs(node2["y_0"] - node1["y_1"]))
                delta_f = abs(node2["font_size"] - node1["font_size"])

                # horizontal edges
                if self.fulfills_horizontal_edge_conditions(node1, node2, page,
                                                            delta_f, length_x_min, delta_y_min):
                    if node1["pos_x"] < node2["pos_x"]:
                        G = self.get_horizontal_edge(G, id1, id2, length_x_min)
                    else:
                        G = self.get_horizontal_edge(G, id2, id1, length_x_min)

                # vertical edges
                if self.fulfills_vertical_edge_conditions(node1, node2, page,
                                                          delta_x_min, length_y_min, delta_f):
                    if node2["y_1"] > node1["y_1"]:
                        G = self.get_vertical_edge(G, id1, id2, length_y_min)
                    else:
                        G = self.get_vertical_edge(G, id2, id1, length_y_min)
        return G

    def get_horizontal_edge(self, G, id_start, id_end, length):
        """

        :param G:
        :param id_start:
        :param id_end:
        :param length:
        :return:
        """
        edges_from_node = [e for e in G.out_edges(id_start, keys=True, data=True)
                           if e[3]["direction"] == "h"]
        if len(edges_from_node) == 0:
            G.add_edge(id_start, id_end, length=length,
                       lengthx_phys=length, lengthy_phys=0, direction="h",
                       weight=length, key="h")
        elif len(edges_from_node) == 1:
            if edges_from_node[0][3]["lengthx_phys"] >= length:
                G.remove_edges_from(edges_from_node)
                G.add_edge(id_start, id_end, length=length,
                           lengthx_phys=length, lengthy_phys=0, direction="h",
                           weight=length, key="h")
        return G

    def get_vertical_edge(self, G, id_start, id_end, length):
        """

        :param G:
        :param id_start:
        :param id_end:
        :param length:
        :return:
        """
        edges_from_node_phys = [e for e in G.out_edges(id_start, keys=True, data=True)
                                if e[3]["direction"] == "v"]
        if len(edges_from_node_phys) == 0:
            G.add_edge(id_start, id_end, length=length,
                       lengthx_phys=0, lengthy_phys=length,
                       direction="v", weight=length, key="v")
        elif len(edges_from_node_phys) == 1:
            if edges_from_node_phys[0][3]["lengthy_phys"] >= length:
                G.remove_edges_from(edges_from_node_phys)
                G.add_edge(id_start, id_end, length=length,
                           lengthx_phys=0, lengthy_phys=length,
                           direction="v", weight=length, key="v")
        return G

    def label_loops(self, graph):
        """

        :param graph:
        :return:
        """
        loop_labels = dict()
        all_nodes = dict(graph.nodes(data=True))
        subgraph_H = nx.DiGraph()
        subgraph_V = nx.DiGraph()
        subgraph_H.add_nodes_from(graph)
        subgraph_V.add_nodes_from(graph)
        subgraph_H.add_edges_from([(e[0], e[1]) for e in graph.edges(keys=True, data=True) if e[2] == "h"])
        subgraph_V.add_edges_from([(e[0], e[1]) for e in graph.edges(keys=True, data=True) if e[2] == "v"])
        distances = dict(nx.all_pairs_shortest_path_length(subgraph_V, cutoff=3))
        nodes_list = list(subgraph_V.nodes)
        num_nodes = len(subgraph_V)
        A_V = np.zeros((num_nodes, num_nodes))
        for i in range(num_nodes):
            id1 = nodes_list[i]
            if id1 in distances:
                for j in range(num_nodes):
                    id2 = nodes_list[j]
                    if id2 in distances[id1] and id1 != id2:
                        A_V[i, j] = 1
        distances = dict(nx.all_pairs_shortest_path_length(subgraph_H, cutoff=3))
        nodes_list = list(subgraph_H.nodes)
        num_nodes = len(subgraph_H)
        A_H = np.zeros((num_nodes, num_nodes))
        for i in range(num_nodes):
            id1 = nodes_list[i]
            if id1 in distances:
                for j in range(num_nodes):
                    id2 = nodes_list[j]
                    if id2 in distances[id1] and id1 != id2:
                        A_H[i, j] = 1

        loops = np.argwhere((A_H.dot(A_V) + A_V.dot(A_H) == 2) |
                            (A_H.transpose().dot(A_V) + A_V.dot(A_H.transpose()) == 2))
        loop_vertices = list(set(loops.flatten()))
        for x in loops:
            edges_from_node = [e for e in graph.out_edges(x[0], keys=True, data=True)
                               if e[3]["direction"] == "l"]
            lengthx = min(abs(all_nodes[x[0]]["x_0"]-all_nodes[x[1]]["x_1"]), abs(all_nodes[x[0]]["x_1"]-all_nodes[x[1]]["x_0"]))
            lengthy = min(abs(all_nodes[x[0]]["y_0"] - all_nodes[x[1]]["y_1"]), abs(all_nodes[x[0]]["y_1"] - all_nodes[x[1]]["y_0"]))
            length = np.sqrt(lengthx**2 + lengthy**2)
            if len(edges_from_node) == 0:
                graph.add_edge(x[0], x[1], length=length,
                               lengthx_phys=lengthx, lengthy_phys=lengthy, direction="l",
                               weight=1 / length, key="l")
            elif len(edges_from_node) > 0:
                if edges_from_node[0][3]["length"] >= length:
                    graph.remove_edges_from(edges_from_node)
                    graph.add_edge(x[0], x[1], length=length,
                                   lengthx_phys=lengthx, lengthy_phys=lengthy, direction="l",
                                   weight=1 / length, key="l")
        loop_labels.update(dict.fromkeys(loop_vertices, 1))
        nx.set_node_attributes(graph, loop_labels, 'is_loop')
        return graph

    def fulfills_horizontal_edge_conditions(self, node1, node2, page,
                                            delta_f, length_x_min, delta_y_min):
        """

        :param node1:
        :param node2:
        :param page:
        :param delta_f:
        :param length_x_min:
        :param delta_y_min:
        :return:
        """
        return (delta_y_min < self.meta.y_eps or
                (constants.USE_VERTICAL_OVERLAP and
                 GraphConverter.is_vertically_overlapping(node1, node2))) and (
                (length_x_min < self.meta.threshold_x[page] and
                 (not constants.USE_FONT or
                  delta_f < self.meta.font_eps_h
                  )) or
                (constants.USE_RECT and
                 RectangleUtil.is_same_rectangle(node1, node2)))

    def fulfills_vertical_edge_conditions(self, node1, node2, page,
                                          delta_x_min, length_y_min, delta_f):
        """

        :param node1:
        :param node2:
        :param page:
        :param delta_x_min:
        :param length_y_min:
        :param delta_f:
        :return:
        """
        width_1 = node1["x_1"] - node1["x_0"]
        width_2 = node2["x_1"] - node2["x_0"]
        width_rel = abs(width_1 - width_2) / max(width_1, width_2, 1)
        return (delta_x_min < self.meta.x_eps or
                (constants.USE_HORIZONTAL_OVERLAP and
                 GraphConverter.is_horizontally_overlapping(node1, node2))) and\
                (length_y_min < self.meta.threshold_y[page] and
                 (not constants.USE_WIDTH or
                  ((width_rel < self.meta.width_pct_eps or
                    max(width_1, width_2) < self.meta.threshold_page_width[page])))
                 and (delta_f < self.meta.font_eps_v or not constants.USE_FONT)
                 or (constants.USE_RECT and
                     RectangleUtil.is_same_rectangle(node1, node2)))

    @staticmethod
    def is_horizontally_overlapping(node1, node2):
        """

        :param node1:
        :param node2:
        :return:
        """
        union = max(node2["x_1"]-node1["x_0"], node1["x_1"]-node2["x_0"])
        intersection = min(node1["x_1"], node2["x_1"]) - max(node1["x_0"], node2["x_0"])
        return intersection / union >= constants.HORIZONTAL_OVERLAP_THRESHOLD

    @staticmethod
    def is_vertically_overlapping(node1, node2):
        """

        :param node1:
        :param node2:
        :return:
        """
        min_height = min(node1["y_0"] - node1["y_1"], node2["y_0"] - node2["y_1"])
        overlap = min(node1["y_0"], node2["y_0"]) - max(node1["y_1"], node2["y_1"])
        return overlap / min_height >= constants.VERTICAL_OVERLAP_THRESHOLD

    def get_media_boxes(self):
        """

        :return:
        """
        return self.media_boxes

    def get_page_count(self):
        """

        :return:
        """
        return self.n



import pickle


def save_object(obj, path, name):
    """

    :param obj:
    :param path:
    :param name:
    :return:
    """
    with open(path + name + '.pkl', 'wb') as f:
        pickle.dump(obj, f, pickle.HIGHEST_PROTOCOL)


def load_object(path, name):
    """

    :param path:
    :param name:
    :return:
    """
    with open(path + name + '.pkl', 'rb') as f:
        return pickle.load(f)


def get_file_name(path):
    """

    :param path:
    :return:
    """
    parts = path.split("/")
    return parts[len(parts) - 1]


def replace_file_type(file_name, new_type):
    """

    :param file_name:
    :param new_type:
    :return:
    """
    file_name_parts = file_name.split(".")
    return file_name.replace(file_name_parts[len(file_name_parts)-1], new_type)


def cut_file_type(file_name):
    """

    :param file_name:
    :return:
    """
    file_name_parts = file_name.split(".")
    return file_name.replace("." + file_name_parts[len(file_name_parts)-1], "")




def in_same_element_type(node1, node2):
    """

    :param node1:
    :param node2:
    :return:
    """
    return node1["in_element"] == node2["in_element"]


def is_same_rectangle(node1, node2, use_ids=False):
    """

    :param node1:
    :param node2:
    :param use_ids:
    :return:
    """
    return (node1["in_element"] == "rectangle" and
            node1["in_element_ids"] == node2["in_element_ids"] and
            use_ids) or node1["box"] == node2["box"]


def is_sharing_vertical_border(node1, node2):
    """

    :param node1:
    :param node2:
    :return:
    """
    return ((node1["in_element_ids"] == [] and node2["in_element_ids"] == []) or (
            node1["in_element_ids"] != [] and node2["in_element_ids"] != [] and (
            (node1["in_element_ids"][1] == node2["in_element_ids"][0]) or
            (node1["in_element_ids"][0] == node2["in_element_ids"][1]))))


def is_sharing_horizontal_border(node1, node2):
    """

    :param node1:
    :param node2:
    :return:
    """
    return ((node1["in_element_ids"] == [] and node2["in_element_ids"] == []) or (
            node1["in_element_ids"] != [] and node2["in_element_ids"] != [] and (
            (node1["in_element_ids"][2] == node2["in_element_ids"][3]) or
            (node1["in_element_ids"][3] == node2["in_element_ids"][2]))))



import numpy as np
from GraphConverter.util import StorageUtil, constants


class DocumentMetaCharacteristics(object):

    x_var = 0
    y_var = 0
    font_size_var = 0
    bold_pct = 0
    italic_pct = 0
    font_name_entropy = 0
    font_size_entropy = 0
    mean_x_diff_page = []

    def __init__(self, locations, media_boxes, optimize=True):
        self.page_ratio_x = constants.DEFAULT_PAGE_RATIO_X
        self.page_ratio_y = constants.DEFAULT_PAGE_RATIO_Y
        self.x_eps = constants.DEFAULT_X_EPS
        self.y_eps = constants.DEFAULT_Y_EPS
        self.x_phys = constants.DEFAULT_X_PHYS
        self.font_eps_h = constants.DEFAULT_FONT_EPS_HORIZONTAL
        self.font_eps_v = constants.DEFAULT_FONT_EPS_VERTICAL
        self.width_pct_eps = constants.DEFAULT_WIDTH_PCT
        self.width_page_eps = constants.DEFAULT_PAGE_WIDTH_EPS
        self.avg_widths = []
        self.x_var_page = []
        self.y_var_page = []
        self.x_diff_page = []
        self.mean_x_diff_page = []
        self.mean_y_diff_page = []
        self.y_diff_page = []
        self.locations = locations
        self.media_boxes = media_boxes
        self.optimize_params = optimize
        self.num_pages = int(np.max(self.locations["page"])) + 1
        self.threshold_x = np.full(self.num_pages, 250)
        self.threshold_y = np.full(self.num_pages, 150)
        self.media_box_width = np.full(self.num_pages, 600)
        self.threshold_page_width = np.full(self.num_pages, 600)
        self.font_name_entropy = 0
        self.font_size_entropy = 0

    def optimize(self):
        """

        :return:
        """
        # x and y coordinate difference deviations
        for page in self.locations["page"].unique():
            page_loc = self.locations[self.locations["page"] == page]
            xs = sorted(page_loc["x_0"].unique())
            xs_diff = [j-i for i, j in zip(xs[:-1], xs[1:])]
            ys = sorted(page_loc["y_0"].unique())
            ys_diff = [j-i for i, j in zip(ys[:-1], ys[1:])]
            # x0 and y0 differences + means + standard deviations
            self.x_diff_page.append(xs_diff)
            self.mean_x_diff_page.append(np.mean(xs_diff) if len(xs_diff) > 0 else 0)
            self.x_var_page.append(np.std(xs_diff) if len(xs_diff) > 1 else 0)
            self.y_diff_page.append(ys_diff)
            self.mean_y_diff_page.append(np.mean(ys_diff) if len(ys_diff) > 0 else 0)
            self.y_var_page.append(np.std(ys_diff) if len(ys_diff) > 1 else 0)
            # average widths
            self.avg_widths.append((page_loc["x_1"] - page_loc["x_0"]).mean())
        # coordinate difference standard deviation means
        self.x_var = np.mean(self.x_var_page)
        self.y_var = np.mean(self.y_var_page)
        # font size deviations
        self.font_size_var = np.std(self.locations["font_size"])
        font_size_pct = self.locations["font_size"].value_counts() / len(self.locations)
        if len(font_size_pct) == 1:
            self.font_size_entropy = 0
        else:
            for pct in font_size_pct:
                self.font_size_entropy -= pct * np.log(pct) / np.log(len(font_size_pct))
        # bold + italic percentages
        self.bold_pct = np.mean(self.locations["bold"])
        self.italic_pct = np.mean(self.locations["italic"])
        # font entropy
        font_name_pct = self.locations["font_name"].value_counts() / len(self.locations)
        if len(font_name_pct) == 1:
            self.font_name_entropy = 0
        else:
            for pct in font_name_pct:
                self.font_name_entropy -= pct * np.log(pct) / np.log(len(font_name_pct))

    def regress_parameters(self):
        """

        :return:
        """
        X = [[self.font_size_entropy, self.font_name_entropy, self.bold_pct,
              self.italic_pct, self.x_var, self.y_var, np.mean(self.avg_widths)]]
        self.x_eps = constants.DEFAULT_X_EPS
        self.y_eps = constants.DEFAULT_Y_EPS
        self.x_phys = constants.DEFAULT_X_PHYS
        self.font_eps_h = constants.DEFAULT_FONT_EPS_HORIZONTAL
        self.font_eps_v = constants.DEFAULT_FONT_EPS_VERTICAL
        self.width_pct_eps = constants.DEFAULT_WIDTH_PCT
        self.width_page_eps = 1
        self.page_ratio_x = StorageUtil.load_object(constants.MODEL_PATH, "page_ratio_x").predict(X)
        self.page_ratio_y = StorageUtil.load_object(constants.MODEL_PATH, "page_ratio_y").predict(X)
        self.x_eps = StorageUtil.load_object(constants.MODEL_PATH, "x_eps").predict(X)
        self.x_phys = self.x_eps
        self.y_eps = StorageUtil.load_object(constants.MODEL_PATH, "page_ratio_y").predict(X)
        self.font_eps_h = StorageUtil.load_object(constants.MODEL_PATH, "font_eps").predict(X)
        self.font_eps_v = self.font_eps_h
        self.width_pct_eps = StorageUtil.load_object(constants.MODEL_PATH, "width_pct_eps").predict(X)
        self.width_page_eps = StorageUtil.load_object(constants.MODEL_PATH, "threshold_page_width").predict(X)

    def generate_attributes(self):
        """

        :return:
        """
        self.optimize()  # calculate meta characteristics
        if self.optimize_params:
            self.regress_parameters()
        # default thresholds based on page size
        for page in range(self.num_pages):
            self.media_box_width[page] = self.media_boxes[page]['x1'] - \
                                         self.media_boxes[page]['x0']
            self.threshold_page_width[page] = self.width_page_eps * self.media_box_width[page]
            self.threshold_x[page] = abs(self.media_boxes[0]['x1'] -
                                         self.media_boxes[0]['x0']) / self.page_ratio_x
            self.threshold_y[page] = abs(self.media_boxes[0]['y1'] -
                                         self.media_boxes[0]['y0']) / self.page_ratio_y




# text box transformation
MERGE_RECTANGLE_TEXT_BOXES = False
USE_RECTANGLE_BOX_COORDINATES = False

# paths
PDF_PATH = "./pdf/"
MODEL_PATH = "GraphConverter/models/graph_parameters/"

# document meta calculations
OPTIMIZE = False

# visibility graph settings
USE_FONT = True
USE_WIDTH = True
USE_RECT = True
USE_HORIZONTAL_OVERLAP = False
USE_VERTICAL_OVERLAP = False
HORIZONTAL_OVERLAP_THRESHOLD = 0.3
VERTICAL_OVERLAP_THRESHOLD = 0.7

# visibility graph parameters
DEFAULT_PAGE_RATIO_X = 1.9
DEFAULT_PAGE_RATIO_Y = 20
DEFAULT_X_EPS = 2
DEFAULT_Y_EPS = 2
DEFAULT_X_PHYS = DEFAULT_X_EPS
DEFAULT_FONT_EPS_VERTICAL = 1
DEFAULT_FONT_EPS_HORIZONTAL = DEFAULT_FONT_EPS_VERTICAL
DEFAULT_WIDTH_PCT = .4
DEFAULT_PAGE_WIDTH_EPS = .5



import pandas as pd
from GraphConverter.util import constants


class PDFTextBoxMerging(object):

    def __init__(self, data, media_boxes):
        self.data = data
        self.media_boxes = media_boxes

    def transform(self):
        """

        :return:
        """
        if constants.USE_RECTANGLE_BOX_COORDINATES:
            self.data = self.transform_text_to_box_coordinates()
        if constants.MERGE_RECTANGLE_TEXT_BOXES:
            self.data = self.merge_elements()
        return self.data

    def transform_text_to_box_coordinates(self):
        """

        :return:
        """
        return self.data.apply(lambda x: self.text_to_box_coordinates(x), axis=1)

    def text_to_box_coordinates(self, row):
        """

        :param row:
        :return:
        """
        if row["in_element_ids"].count(-1) == 0:
            row["x_0"] = row["in_element_ids"][0]
            row["y_1"] = row["in_element_ids"][1]
            row["y_0"] = row["in_element_ids"][3]
            row["x_1"] = row["in_element_ids"][2]
            row["pos_x"] = (row["x_0"] + row["x_1"]) / 2
            row["pos_y"] = (row["y_0"] + row["y_1"]) / 2
        return row

    def merge_elements(self):
        """

        :return:
        """
        self.data["in_element_ids2"] = self.data["in_element_ids"].apply(str)
        self.data = self.data.sort_values(by=['page', 'in_element_ids2', 'box'])
        data_new = pd.DataFrame(data=None, columns=self.data.columns,
                                index=self.data.index)
        prev_rect = ""
        row_complete = None
        add = False
        for i, row in self.data.iterrows():
            if row["in_element"] != "rectangle" or row["in_element_ids"].count(-1) != 0:
                data_new = data_new.append(row, ignore_index=True)
            else:
                rect = row["in_element_ids2"]
                if rect == prev_rect:
                    row_complete["text"] = row_complete["text"] + "\n" + row["text"]
                    if not constants.USE_RECTANGLE_BOX_COORDINATES:
                        row_complete["x_0"] = min(row["x_0"], row_complete["x_0"])
                        row_complete["y_1"] = min(row["y_1"], row_complete["y_1"])
                        row_complete["y_0"] = max(row["y_0"], row_complete["y_0"])
                        row_complete["x_1"] = max(row["x_1"], row_complete["x_1"])
                    add = True
                else:
                    if row_complete is None:
                        row_complete = row
                    if add and not constants.USE_RECTANGLE_BOX_COORDINATES:
                        page_height = self.media_boxes[int(row_complete["page"])]["y1page"]
                        row_complete["pos_x"] = (row_complete["x_0"] + row_complete["x_1"]) / 2
                        row_complete["pos_y"] = (row_complete["y_0"] + row_complete["y_1"]) / 2
                        row_complete["abs_pos"] = (row_complete["pos_x"],
                                                   page_height - row_complete["pos_y"] -
                                                   page_height * row_complete["page"])
                        add = False
                    data_new = data_new.append(row_complete)
                    row_complete = row
                prev_rect = rect
        data_new.dropna(inplace=True)
        self.data = data_new.drop(["in_element_ids2"], axis=1)
        return self.data



from distutils.core import setup

with open('README.rst') as f:
    long_description = f.read()
print(long_description)
setup(
  name='GraphConverter',
  packages=['GraphConverter', 'GraphConverter.util', 'GraphConverter.document', 'GraphConverter.merging',
            'GraphConverter.models'],
  version='0.3',
  license='MIT',
  description='A tool for creating a graph representation out of the content of PDF documents.',
  long_description=long_description,
  author='Michael Aigner, Florian Preis',
  # author_email='your.email@domain.com',
  url='https://github.com/MBAigner/Graph-Converter',
  download_url='https://github.com/MBAigner/GraphConverter/archive/v0.1.tar.gz',
  keywords=['python', 'pdf', 'pdf-converter', 'graph', 'graph-algorithms', 'graph-representation', 'visibility-graph',
            'document-analysis'],
  install_requires=[
      'numpy',
      'pandas==1.0.3',
      'networkx==2.2',
      'scikit-learn==0.20.0',
      'PDFContentConverter==0.7'
      ],
  classifiers=[
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7'
  ],
)




from GraphConverter.GraphConverter import GraphConverter
from GraphConverter.util import RectangleUtil, constants, StorageUtil
from GraphConverter.document import DocumentMetaCharacteristics
from GraphConverter.merging import PDFTextBoxMerging
